package internal

import (
	"archive/zip"
	"encoding/json"
	"io/ioutil"
	"log"
	"os"
	"regexp"

	"github.com/go-resty/resty/v2"
)

type App struct {
	URL      string `json:"url"`
	Filename string `json:"filename"`
	ID       string `json:"id"`
}

func NewApp(app string) App {
	if IsUrl(app) {
		return App{URL: app}
	}
	return App{Filename: app}
}

func IsUrl(value string) bool {
	ok, _ := regexp.MatchString(`^https?://`, value)
	return ok
}

func (app *App) Fetch() {
	resp, err := resty.New().R().Get(app.URL)
	DieOnError(err)
	DieOnResponseError(resp)
	re := regexp.MustCompile(`filename=\"([\w\d\.]*)\"`)
	groups := re.FindStringSubmatch(resp.Header().Get("content-disposition"))
	if len(groups) != 2 {
		log.Fatal("can't guess filename")
	}
	app.Filename = groups[1]
	err = ioutil.WriteFile(LocalPath(app.Filename), resp.Body(), 0644)
	DieOnError(err)
	log.Printf("downloaded %s", app.Filename)
}

func (app *App) LoadID() {
	r, err := zip.OpenReader(LocalPath(app.Filename))
	DieOnError(err)
	defer r.Close()
	f, err := r.Open("app.json")
	DieOnError(err)
	defer f.Close()
	var j map[string]interface{}
	err = json.NewDecoder(f).Decode(&j)
	DieOnError(err)
	id, ok := j["id"]
	if !ok {
		log.Fatal("no id in app.json")
	}
	app.ID = id.(string)
}

func (app App) FileExist() bool {
	_, err := os.Stat(LocalPath(app.Filename))
	if err != nil {
		return false
	}
	return true
}

func FindApp(apps []App, filter func(App) bool) *App {
	for _, app := range apps {
		if filter(app) {
			return &app
		}
	}
	return nil
}

func FetchAll(names []string, force bool) []App {
	cache := ReadCache()
	apps := []App{}
	for _, v := range names {
		cachedApp := FindApp(cache, func(a App) bool {
			return a.URL == v
		})
		var app App
		if cachedApp != nil {
			app = *cachedApp
		} else {
			app = NewApp(v)
		}
		if app.URL != "" && (app.Filename == "" || !app.FileExist() || force) {
			app.Fetch()
		}
		app.LoadID()
		apps = append(apps, app)
	}
	WriteCache(apps)
	return apps
}
